/*
 ***************************************************************************
 * Ralink Tech Inc.
 * 4F, No. 2 Technology 5th Rd.
 * Science-based Industrial Park
 * Hsin-chu, Taiwan, R.O.C.
 *
 * (c) Copyright, Ralink Technology, Inc.
 *
 * All rights reserved. Ralink's source code is an unpublished work and the
 * use of a copyright notice does not imply otherwise. This source code
 * contains confidential trade secret material of Ralink Tech. Any attemp
 * or participation in deciphering, decoding, reverse engineering or in any
 * way altering the source code is stricitly prohibited, unless the prior
 * written consent of Ralink Technology, Inc. is obtained.
 ***************************************************************************
 *
 * $Id: ralink_gpio.c,v 1.8 2008-06-20 06:13:33 winfred Exp $
 */
#include <linux/init.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/fs.h>
#ifdef CONFIG_RALINK_GPIO_LED
#include <linux/timer.h>
#endif
#include <asm/io.h>
#include <asm/uaccess.h>
#include "ralink_gpio_sc.h"
#include "gconfig.h"
#include <asm/rt2880/surfboardint.h>

/* marklin 20080819 : set RT register */
#define RTMP_IO_WRITE32_T(_A, _R, _V)												\
{																				\
	unsigned int	Val;																\
	Val = readl((void *)((_A) + 0x1000));			\
	writel(_V, (void *)((_A) + (_R)));								\
}

#ifdef  CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
static  devfs_handle_t devfs_handle;
#endif

#define NAME			"ralink_gpio"
#define RALINK_GPIO_DEVNAME	"gpio"
int ralink_gpio_major = 252;
u32 ralink_gpio_intp = 0;
int ralink_gpio_irqnum = 0;
u32 ralink_gpio_edge = 0;
ralink_gpio_reg_info ralink_gpio_info[RALINK_GPIO_NUMBER];
extern unsigned long volatile jiffies;
#ifdef CONFIG_RALINK_GPIO_LED
#define RALINK_LED_DEBUG 0
#define RALINK_GPIO_LED_FREQ (HZ/10)
#if (HAS_INIC && FOR_SITECOM)
unsigned long wps_time2=0;
#define RESET2DEF_TIME_PERIOD    (HZ)
#else
#define RESET2DEF_TIME_PERIOD    (10*HZ)
#endif
struct timer_list ralink_gpio_led_timer;
struct timer_list *reset2def_timer;
void ralink_gpio_notify_user(int usr);
void add_reset2def_job_to_timer(void);
#if FOR_IODATA
struct timer_list *wps_timer;
#endif
#ifdef iNIC_2880
unsigned long iNIC_piodata,iNIC_piodata_tmp;
#endif
#ifdef CONFIG_RAETH_RT3052_P5 
int RT3052_WPS_IN_PROCESS=0;
int RT2880_WPS_IN_PROCESS=0;
#endif
#ifdef ENABLE_HOT_PLUG_RESET
int usb_hotplug_flag = 0;
#endif
ralink_gpio_led_info ralink_gpio_led_data[RALINK_GPIO_NUMBER];
u32 ralink_gpio_led_value = 0;
unsigned long wps_time=0;
struct ralink_gpio_led_status_t {
	int ticks;
	unsigned int ons;
	unsigned int offs;
	unsigned int resting;
	unsigned int times;
} ralink_gpio_led_stat[RALINK_GPIO_NUMBER];
#endif

MODULE_DESCRIPTION("Ralink SoC GPIO Driver");
MODULE_AUTHOR("Winfred Lu <winfred_lu@ralinktech.com.tw>");
MODULE_LICENSE("GPL");
ralink_gpio_reg_info info;

volatile int detect_reset2def,reset2def_start,factory_mode;

/* marklin 20080811 : hw reset */
void ralink_reset(int reset_pin)
{
   unsigned long piodir,piodata;

       piodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
       piodir |= (1L << reset_pin);
       *(volatile u32 *)(RALINK_REG_PIODIR) = cpu_to_le32(piodir);

       piodata = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
       udelay(10);
       piodata &= ~(1L << reset_pin);
       *(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(piodata); 
       mdelay(100); /* G693L : output low 140ms after power on */
       piodata |= (1L << reset_pin);
       *(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(piodata);
}

void ralink_gpio_control(int gpio,int level)
{
   unsigned long piodir,piodata;

       piodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
       piodir |= (1L << gpio);
       *(volatile u32 *)(RALINK_REG_PIODIR) = cpu_to_le32(piodir);
       piodata = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));

       if(level)
          piodata |= (1L << gpio);
       else
          piodata &= ~(1L << gpio);

       ralink_gpio_led_value = piodata;
       *(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(piodata); 
}

static void  reset2def_led_action(unsigned long dummy)
{

#ifdef FOR_COREGA
	//printk(" reset2def_led_action : FOR_COREGA \n");
	//register led data
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].gpio = PWR_LED_GPIO_PIN;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].on = 0;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].off = 0;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].blinks = 0;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].rests = 0;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].times = 0;

	//clear previous led status
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ticks = -1;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ons = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].offs = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].resting = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].times = 0;
#else
	//register led data
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].gpio = PWR_LED_GPIO_PIN;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].on = 2;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].off = 2;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].blinks = 10;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].rests = 0;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].times = 10;

	//clear previous led status
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ticks = -1;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ons = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].offs = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].resting = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].times = 0;
#endif
}
#if (HAS_INIC && FOR_SITECOM)
static void  wps_counter(unsigned long dummy)
{
	if((wps_time2==0) ||  (wps_time2==5))
	{
           ralink_gpio_notify_user(2);
	}
	else if(wps_time2==15)
	{
	   reset2def_led_action(0);
	}
	wps_time2++;
	add_reset2def_job_to_timer();
}
#endif
#if FOR_IODATA
static void  wps_action(unsigned long dummy)
{
     ralink_gpio_notify_user(1);
}
#endif
void send_INT_to_RT3025(int status)
{
	unsigned long idx, tmp,piodata;
		ralink_gpio_led_value = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
#if 1 /* wps use interrupt  */
		ralink_gpio_led_value &= ~(1L << GPIO_CONTROL_2880_PIN1);
		if(status)
			ralink_gpio_led_value |= (1L << GPIO_CONTROL_2880_PIN2);
		else
			ralink_gpio_led_value &= ~(1L << GPIO_CONTROL_2880_PIN2);
      		*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(ralink_gpio_led_value);
		mdelay(10);
		ralink_gpio_led_value |= (1L << GPIO_CONTROL_2880_PIN1);
#else /* wps use pollong */
		if(status)
		{
			ralink_gpio_led_value |= (1L << GPIO_CONTROL_2880_PIN1);
			ralink_gpio_led_value |= (1L << GPIO_CONTROL_2880_PIN2);
		}
		else
		{
			ralink_gpio_led_value &= ~(1L << GPIO_CONTROL_2880_PIN1);
			ralink_gpio_led_value &= ~(1L << GPIO_CONTROL_2880_PIN2);
		}
#endif
      		*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(ralink_gpio_led_value);
}


void add_reset2def_job_to_timer(void)
{
        reset2def_timer->expires = jiffies + RESET2DEF_TIME_PERIOD ;
        reset2def_timer->data = (unsigned long)0;
#if (HAS_INIC && FOR_SITECOM)
        reset2def_timer->function = wps_counter;
#else
        reset2def_timer->function = reset2def_led_action;
#endif
        add_timer(reset2def_timer); 
}

#if FOR_IODATA
void add_wps_job_to_timer(void)
{
        wps_timer->expires = jiffies + (2*HZ) ;
        wps_timer->data = (unsigned long)0;
        wps_timer->function = wps_action;
        add_timer(wps_timer); 
}
#endif
int ralink_gpio_ioctl(struct inode *inode, struct file *file, unsigned int req,
		unsigned long arg)
{
	unsigned long idx, tmp,piodata;
	ralink_gpio_reg_info info;
#ifdef CONFIG_RALINK_GPIO_LED
	ralink_gpio_led_info led;
#endif

	idx = (req >> RALINK_GPIO_DATA_LEN) & 0xFFL;
	req &= RALINK_GPIO_DATA_MASK;

	switch(req) {
        case RALINK_GPIO_PWR_LED:
               if(cpu_to_le32(arg) == 1)        /* blinking */
               {
			//register led data
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].gpio = PWR_LED_GPIO_PIN;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].on = 5;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].off = 5;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].blinks = 10;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].rests = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].times = 10;
		
			//clear previous led status
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ticks = -1;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ons = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].offs = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].resting = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].times = 0;
               }
               else if(cpu_to_le32(arg) == 2)   /* led off */
               {
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].gpio = PWR_LED_GPIO_PIN;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].on = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].off = 1;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].blinks = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].rests = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].times = 0;
		
			//clear previous led status
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ticks = -1;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ons = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].offs = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].resting = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].times = 0;
               }
               else                              /* led on */
               {
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].gpio = PWR_LED_GPIO_PIN;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].on = 1;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].off = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].blinks = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].rests = 0;
			ralink_gpio_led_data[PWR_LED_GPIO_PIN].times = 0;
		
			//clear previous led status
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ticks = -1;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ons = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].offs = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].resting = 0;
			ralink_gpio_led_stat[PWR_LED_GPIO_PIN].times = 0;
               }         
		break; 
	case RALINK_GPIO_FACTORY:
		factory_mode = 1;
		break;
	case RALINK_GPIO_RESET:
		ralink_reset(RALINK_GPIO_RESET_PIN);
		break;
	case RALINK_GPIO_RESET_RT2880:
		ralink_reset(RT3052_RESET_RT2880_PIN);
		break;
	case RALINK_GPIO_WPS_TIME:
		put_user(wps_time, (int __user *)arg);
#if (HAS_INIC && FOR_SITECOM)	
	case RALINK_GPIO_WPS_TIME2:
		put_user(wps_time2, (int __user *)arg);		
		break;
#endif
#ifdef ENABLE_HOT_PLUG_RESET
	case RALINK_GPIO_ENABLE_USB_FLAG:
		usb_hotplug_flag = cpu_to_le32(arg);
		break;
#endif
#ifdef iNIC_2880
	case RALINK_GPIO_GET_CMD_FROM_RT3052:
		tmp = 0x0;
		//tmp |= (iNIC_piodata&(1L << GPIO_CONTROL_2880_PIN1)) != 0 ? 0x01 : 0x0;
		//tmp |= (iNIC_piodata&(1L << GPIO_CONTROL_2880_PIN2)) != 0 ? 0x02 : 0x0;
		tmp |= (iNIC_piodata&(1L << GPIO_CONTROL_2880_PIN3)) != 0 ? 0x04 : 0x0;
		put_user(tmp, (int __user *)arg);
		break;
	case RALINK_GPIO_GET_PID_FROM_USER:
		ralink_gpio_info[GPIO_CONTROL_2880_PIN4].pid = (pid_t)cpu_to_le32(arg);
		break;
#endif
#ifdef CONFIG_RAETH_RT3052_P5 
	case RALINK_GPIO_CONTROL_RT2880:
		tmp = (cpu_to_le32(arg) & 0x00000004);
		piodata = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
		piodata &= RALINK_GPIO_CONTROL_RT2880_MASK;
		//piodata |= (tmp&0x1) != 0 ? (1L << GPIO_CONTROL_2880_PIN1) : 0x0;
		//piodata |= (tmp&0x2) != 0 ? (1L << GPIO_CONTROL_2880_PIN2) : 0x0;	
		piodata |= (tmp&0x4) != 0 ? (1L << GPIO_CONTROL_2880_PIN3) : 0x0;
      		*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(piodata);
		ralink_gpio_led_value = piodata;
		tmp = *(volatile u32 *)(RALINK_REG_PIODATA);
		tmp &= ~(1L << GPIO_CONTROL_2880_PIN4);
      		*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(tmp);
		mdelay(10);
		tmp |= (1L << GPIO_CONTROL_2880_PIN4);
      		*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(tmp);
		break;
	case RALINK_GPIO_RT2880_WPS_START:
		RT2880_WPS_IN_PROCESS = 1;
		RT3052_WPS_IN_PROCESS = 0;
		break;
	case RALINK_GPIO_RT3052_WPS_START:
		RT2880_WPS_IN_PROCESS = 0;
		RT3052_WPS_IN_PROCESS = 1;
		break;
#endif
	case RALINK_GPIO_SET_DIR:
		*(volatile u32 *)(RALINK_REG_PIODIR) = cpu_to_le32(arg);
		break;
	case RALINK_GPIO_READ_DIR:
		tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
		put_user(tmp, (int __user *)arg);
		break;
	case RALINK_GPIO_SET_DIR_IN:
		tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
		tmp &= ~cpu_to_le32(arg);
		*(volatile u32 *)(RALINK_REG_PIODIR) = tmp;
		break;
	case RALINK_GPIO_SET_DIR_OUT:
		tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
		tmp |= cpu_to_le32(arg);
		*(volatile u32 *)(RALINK_REG_PIODIR) = tmp;
		break;
	case RALINK_GPIO_READ: //RALINK_GPIO_READ_INT
		tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
		put_user(tmp, (int __user *)arg);
		break;
	case RALINK_GPIO_WRITE: //RALINK_GPIO_WRITE_INT
		*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(arg);
		break;
	case RALINK_GPIO_SET: //RALINK_GPIO_SET_INT
		*(volatile u32 *)(RALINK_REG_PIOSET) = cpu_to_le32(arg);
		break;
	case RALINK_GPIO_CLEAR: //RALINK_GPIO_CLEAR_INT
		*(volatile u32 *)(RALINK_REG_PIORESET) = cpu_to_le32(arg);
		break;
	case RALINK_GPIO_READ_BIT:
		tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
		if (0L <= idx && idx < RALINK_GPIO_DATA_LEN) {
			tmp = (tmp >> idx) & 1L;
			put_user(tmp, (int __user *)arg);
		}
		else
			return -EINVAL;
		break;
	case RALINK_GPIO_WRITE_BIT:
		if (0L <= idx && idx < RALINK_GPIO_DATA_LEN) {
			tmp =le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
			if (arg & 1L)
				tmp |= (1L << idx);
			else
				tmp &= ~(1L << idx);
			*(volatile u32 *)(RALINK_REG_PIODATA)= cpu_to_le32(tmp);
		}
		else
			return -EINVAL;
		break;
	case RALINK_GPIO_READ_BYTE:
		tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
		if (0L <= idx && idx < RALINK_GPIO_DATA_LEN/8) {
			tmp = (tmp >> idx*8) & 0xFFL;
			put_user(tmp, (int __user *)arg);
		}
		else
			return -EINVAL;
		break;
	case RALINK_GPIO_WRITE_BYTE:
		if (0L <= idx && idx < RALINK_GPIO_DATA_LEN/8) {
			tmp =le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
			tmp &= ~(0xFFL << idx*8);
			tmp |= ((arg & 0xFFL) << idx*8);
			*(volatile u32 *)(RALINK_REG_PIODATA)= cpu_to_le32(tmp);
		}
		else
			return -EINVAL;
		break;
	case RALINK_GPIO_ENABLE_INTP:
                if(factory_mode != 1)
                {
		  *(volatile u32 *)(RALINK_REG_INTENA) = cpu_to_le32(RALINK_INTCTL_PIO);
		}
		break;
	case RALINK_GPIO_DISABLE_INTP:
		*(volatile u32 *)(RALINK_REG_INTDIS) = cpu_to_le32(RALINK_INTCTL_PIO);
		break;
	case RALINK_GPIO_REG_IRQ:
             if(factory_mode != 1)
             {
		copy_from_user(&info, (ralink_gpio_reg_info *)arg, sizeof(info));
		if (0 <= info.irq && info.irq < RALINK_GPIO_NUMBER) {
			ralink_gpio_info[info.irq].pid = info.pid;
			tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIORENA));
			tmp |= (0x1 << info.irq);
			*(volatile u32 *)(RALINK_REG_PIORENA) = cpu_to_le32(tmp);
			tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIOFENA));
			tmp |= (0x1 << info.irq);
			*(volatile u32 *)(RALINK_REG_PIOFENA) = cpu_to_le32(tmp);
		}
		else
			printk(KERN_ERR NAME ": irq number(%d) out of range\n",
					info.irq);
	      }
		break;
	case RALINK_GPIO_LED_SET:
#ifdef CONFIG_RALINK_GPIO_LED
		copy_from_user(&led, (ralink_gpio_led_info *)arg, sizeof(led));
		if (0 <= led.gpio && led.gpio < RALINK_GPIO_NUMBER) {
			if (led.on > RALINK_GPIO_LED_INFINITY)
				led.on = RALINK_GPIO_LED_INFINITY;
			if (led.off > RALINK_GPIO_LED_INFINITY)
				led.off = RALINK_GPIO_LED_INFINITY;
			if (led.blinks > RALINK_GPIO_LED_INFINITY)
				led.blinks = RALINK_GPIO_LED_INFINITY;
			if (led.rests > RALINK_GPIO_LED_INFINITY)
				led.rests = RALINK_GPIO_LED_INFINITY;
			if (led.times > RALINK_GPIO_LED_INFINITY)
				led.times = RALINK_GPIO_LED_INFINITY;
			if (led.on == 0 && led.off == 0 && led.blinks == 0 &&
					led.rests == 0) {
				ralink_gpio_led_data[led.gpio].gpio = -1; //stop it
				break;
			}
			//register led data
			ralink_gpio_led_data[led.gpio].gpio = led.gpio;
			ralink_gpio_led_data[led.gpio].on = led.on;
			ralink_gpio_led_data[led.gpio].off = led.off;
			ralink_gpio_led_data[led.gpio].blinks = led.blinks;
			ralink_gpio_led_data[led.gpio].rests = led.rests;
			ralink_gpio_led_data[led.gpio].times = led.times;

			//clear previous led status
			ralink_gpio_led_stat[led.gpio].ticks = -1;
			ralink_gpio_led_stat[led.gpio].ons = 0;
			ralink_gpio_led_stat[led.gpio].offs = 0;
			ralink_gpio_led_stat[led.gpio].resting = 0;
			ralink_gpio_led_stat[led.gpio].times = 0;

			//set gpio direction to 'out'
			tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
			tmp |= RALINK_GPIO(led.gpio);
			*(volatile u32 *)(RALINK_REG_PIODIR) = tmp;
#if RALINK_LED_DEBUG
			printk("dir_%x gpio_%d - %d %d %d %d %d\n", tmp,
					led.gpio, led.on, led.off, led.blinks,
					led.rests, led.times);
#endif
		}
		else
			printk(KERN_ERR NAME ": gpio number(%d) out of range\n",
					led.gpio);
#else
		printk(KERN_ERR NAME ": gpio led support not built\n");
#endif
		break;
	default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

int ralink_gpio_open(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	MOD_INC_USE_COUNT;
#else
	try_module_get(THIS_MODULE);
#endif
	return 0;
}

int ralink_gpio_release(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	MOD_DEC_USE_COUNT;
#else
	module_put(THIS_MODULE);
#endif
	return 0;
}

struct file_operations ralink_gpio_fops =
{
	owner:		THIS_MODULE,
	ioctl:		ralink_gpio_ioctl,
	open:		ralink_gpio_open,
	release:	ralink_gpio_release,
};

#ifdef CONFIG_RALINK_GPIO_LED

#if RALINK_GPIO_LED_LOW_ACT
#define __LED_ON(gpio) ralink_gpio_led_value &= ~RALINK_GPIO(gpio);
#define __LED_OFF(gpio) ralink_gpio_led_value |= RALINK_GPIO(gpio);
#else
#define __LED_ON(gpio) ralink_gpio_led_value |= RALINK_GPIO(gpio);
#define __LED_OFF(gpio) ralink_gpio_led_value &= ~RALINK_GPIO(gpio);
#endif

static void ralink_gpio_led_do_timer(unsigned long unused)
{
	int i;
	unsigned int x,k=0;
#ifdef CONFIG_RAETH_RT3052_P5
#if 0 /* wps use polling  */
	if(RT2880_WPS_IN_PROCESS == 1)
	   RT3052_WPS_IN_PROCESS = 0;
	if(((*(volatile u32 *)(RALINK_REG_PIODATA)) & (1L<<GPIO_CONTROL_2880_PIN1)) == 0x0)
	{/* RT2880 in the WPS process */

		if(RT3052_WPS_IN_PROCESS == 0)
		{
		   if(((*(volatile u32 *)(RALINK_REG_PIODATA)) & (1L<<GPIO_CONTROL_2880_PIN2)) == 0x0)
		     RT2880_WPS_IN_PROCESS = 1;
		}
		else
		    RT2880_WPS_IN_PROCESS = 0;
	}
#endif
	if(RT2880_WPS_IN_PROCESS == 1)
	{
	    if((*(volatile u32 *)(RALINK_REG_PIODATA)) & (1L<<GPIO_CONTROL_2880_PIN2))
	    {
	       __LED_ON(WPS_LED_GPIO_PIN);
	    }
	    else
	    {
	       __LED_OFF(WPS_LED_GPIO_PIN);
 	    }
	    *(volatile u32 *)(RALINK_REG_PIODATA) = ralink_gpio_led_value;
	}
#endif
	for (i = 0; i < RALINK_GPIO_NUMBER; i++) {
		ralink_gpio_led_stat[i].ticks++;
		if (ralink_gpio_led_data[i].gpio == -1) //-1 means unused
			continue;
		k++;
		if(k==1)
	           ralink_gpio_led_value = *(volatile u32 *)(RALINK_REG_PIODATA);
		if (ralink_gpio_led_data[i].on == RALINK_GPIO_LED_INFINITY ||
				ralink_gpio_led_data[i].off == 0) { //always on
			__LED_ON(i);
			continue;
		}
		if (ralink_gpio_led_data[i].off == RALINK_GPIO_LED_INFINITY ||
				ralink_gpio_led_data[i].rests == RALINK_GPIO_LED_INFINITY ||
				ralink_gpio_led_data[i].on == 0 ||
				ralink_gpio_led_data[i].blinks == 0 ||
				ralink_gpio_led_data[i].times == 0) { //always off
			__LED_OFF(i);
			continue;
		}

		//led turn on or off
		if (ralink_gpio_led_data[i].blinks == RALINK_GPIO_LED_INFINITY ||
				ralink_gpio_led_data[i].rests == 0) { //always blinking
			x = ralink_gpio_led_stat[i].ticks % (ralink_gpio_led_data[i].on
					+ ralink_gpio_led_data[i].off);
		}
		else {
			unsigned int a, b, c, d, o, t;
			a = ralink_gpio_led_data[i].blinks / 2;
			b = ralink_gpio_led_data[i].rests / 2;
			c = ralink_gpio_led_data[i].blinks % 2;
			d = ralink_gpio_led_data[i].rests % 2;
			o = ralink_gpio_led_data[i].on + ralink_gpio_led_data[i].off;
			//t = blinking ticks
			t = a * o + ralink_gpio_led_data[i].on * c;
			//x = ticks % (blinking ticks + resting ticks)
			x = ralink_gpio_led_stat[i].ticks %
				(t + b * o + ralink_gpio_led_data[i].on * d);
			//starts from 0 at resting cycles
			if (x >= t)
				x -= t;
			x %= o;
		}
		if (x < ralink_gpio_led_data[i].on) {
			__LED_ON(i);
			if (ralink_gpio_led_stat[i].ticks && x == 0)
				ralink_gpio_led_stat[i].offs++;
#if RALINK_LED_DEBUG
			printk("t%d gpio%d on,", ralink_gpio_led_stat[i].ticks, i);
#endif
		}
		else {
			__LED_OFF(i);
			if (x == ralink_gpio_led_data[i].on)
				ralink_gpio_led_stat[i].ons++;
#if RALINK_LED_DEBUG
			printk("t%d gpio%d off,", ralink_gpio_led_stat[i].ticks, i);
#endif
		}

		//blinking or resting
		if (ralink_gpio_led_data[i].blinks == RALINK_GPIO_LED_INFINITY ||
				ralink_gpio_led_data[i].rests == 0) { //always blinking
			continue;
		}
		else {
			x = ralink_gpio_led_stat[i].ons + ralink_gpio_led_stat[i].offs;
			if (!ralink_gpio_led_stat[i].resting) {
				if (x == ralink_gpio_led_data[i].blinks) {
					ralink_gpio_led_stat[i].resting = 1;
					ralink_gpio_led_stat[i].ons = 0;
					ralink_gpio_led_stat[i].offs = 0;
					ralink_gpio_led_stat[i].times++;
				}
			}
			else {
				if (x == ralink_gpio_led_data[i].rests) {
					ralink_gpio_led_stat[i].resting = 0;
					ralink_gpio_led_stat[i].ons = 0;
					ralink_gpio_led_stat[i].offs = 0;
				}
			}
		}
		if (ralink_gpio_led_stat[i].resting) {
			__LED_OFF(i);
#if RALINK_LED_DEBUG
			printk("resting,");
		} else {
			printk("blinking,");
#endif
		}

		//number of times
		if (ralink_gpio_led_data[i].times != RALINK_GPIO_LED_INFINITY)
		{
			if (ralink_gpio_led_stat[i].times ==
					ralink_gpio_led_data[i].times) {
				__LED_OFF(i);
				ralink_gpio_led_data[i].gpio = -1; //stop
			}
#if RALINK_LED_DEBUG
			printk("T%d\n", ralink_gpio_led_stat[i].times);
		} else {
			printk("T@\n");
#endif
		}
	}
#if 0
	//always turn the power LED on
#ifdef CONFIG_RALINK_RT2880
	__LED_ON(12);
#elif defined (CONFIG_RALINK_RT3052) || defined (CONFIG_RALINK_RT2883)
	__LED_ON(9);
#endif
#endif
	if(k!=0)
	{
	   *(volatile u32 *)(RALINK_REG_PIODATA) = ralink_gpio_led_value;
	}
#if RALINK_LED_DEBUG
	printk("led_value= %x\n", ralink_gpio_led_value);
#endif

	init_timer(&ralink_gpio_led_timer);
	ralink_gpio_led_timer.expires = jiffies + RALINK_GPIO_LED_FREQ;
	add_timer(&ralink_gpio_led_timer);
}

void ralink_gpio_led_init_timer(void)
{
	int i;

#if 0
	for (i = 0; i < RALINK_GPIO_NUMBER; i++) {
		ralink_gpio_led_data[i].gpio = -1; //-1 means unused
#if RALINK_GPIO_LED_LOW_ACT
		ralink_gpio_led_value |= RALINK_GPIO(i);
#else
		ralink_gpio_led_value &= ~RALINK_GPIO(i);
#endif
	}
#else
	for (i = 0; i < RALINK_GPIO_NUMBER; i++) {
		ralink_gpio_led_data[i].gpio = -1; //-1 means unused
	}
	ralink_gpio_led_value = *(volatile u32 *)(RALINK_REG_PIODATA);
	ralink_gpio_led_value = RALINK_GPIO(WPS_LED_GPIO_PIN);
#endif
	init_timer(&ralink_gpio_led_timer);
	ralink_gpio_led_timer.function = ralink_gpio_led_do_timer;
	ralink_gpio_led_timer.expires = jiffies + RALINK_GPIO_LED_FREQ;
	add_timer(&ralink_gpio_led_timer);
}
#endif

int __init ralink_gpio_init(void)
{
	unsigned int i;
	u32 gpiomode;
	unsigned long tmp;

#ifdef  CONFIG_DEVFS_FS
	if (devfs_register_chrdev(ralink_gpio_major, RALINK_GPIO_DEVNAME,
				&ralink_gpio_fops)) {
		printk(KERN_ERR NAME ": unable to register character device\n");
		return -EIO;
	}
	devfs_handle = devfs_register(NULL, RALINK_GPIO_DEVNAME,
			DEVFS_FL_DEFAULT, ralink_gpio_major, 0,
			S_IFCHR | S_IRUGO | S_IWUGO, &ralink_gpio_fops, NULL);
#else
	int r = 0;
	r = register_chrdev(ralink_gpio_major, RALINK_GPIO_DEVNAME,
			&ralink_gpio_fops);
	if (r < 0) {
		printk(KERN_ERR NAME ": unable to register character device\n");
		return r;
	}
	if (ralink_gpio_major == 0) {
		ralink_gpio_major = r;
		printk(KERN_DEBUG NAME ": got dynamic major %d\n", r);
	}
#endif

	//config these pins to gpio mode
	gpiomode = le32_to_cpu(*(volatile u32 *)(RALINK_REG_GPIOMODE));
	gpiomode |= RALINK_GPIOMODE_DFT;
	*(volatile u32 *)(RALINK_REG_GPIOMODE) = cpu_to_le32(gpiomode);

	//enable gpio interrupt
	*(volatile u32 *)(RALINK_REG_INTENA) = cpu_to_le32(RALINK_INTCTL_PIO);
	for (i = 0; i < RALINK_GPIO_NUMBER; i++) {
		ralink_gpio_info[i].irq = i;
		ralink_gpio_info[i].pid = 0;
	}

	detect_reset2def = 0;
	reset2def_start = 0;
	factory_mode = 0;
        reset2def_timer = (struct timer_list*)kmalloc(sizeof(struct timer_list),GFP_KERNEL);
        init_timer(reset2def_timer);
#if FOR_IODATA
        wps_timer = (struct timer_list*)kmalloc(sizeof(struct timer_list),GFP_KERNEL);
        init_timer(wps_timer);
#endif
#ifdef CONFIG_RAETH_RT3052_P5
#if 1 /* wps use interrupt  */
	*(volatile u32 *)(RALINK_REG_INTENA) = cpu_to_le32(RALINK_INTCTL_PIO);
	ralink_gpio_info[GPIO_CONTROL_2880_PIN1].pid = 0x0;
	tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIOFENA));
	tmp |= (0x1 << GPIO_CONTROL_2880_PIN1);
	*(volatile u32 *)(RALINK_REG_PIOFENA) = cpu_to_le32(tmp);
#endif
       tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
#if 0 /* wps use polling  */
       tmp &= ~(1L << GPIO_CONTROL_2880_PIN1);
#endif
       tmp |= (1L << GPIO_CONTROL_2880_PIN3);
       tmp |= (1L << GPIO_CONTROL_2880_PIN4);
       tmp &= ~(1L << GPIO_CONTROL_2880_PIN2);
       *(volatile u32 *)(RALINK_REG_PIODIR) = cpu_to_le32(tmp);

       tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
       tmp |= (1L << GPIO_CONTROL_2880_PIN3);
       tmp |= (1L << GPIO_CONTROL_2880_PIN4);
      	*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(tmp);
#endif
#ifdef iNIC_2880
	*(volatile u32 *)(RALINK_REG_INTENA) = cpu_to_le32(RALINK_INTCTL_PIO);
	ralink_gpio_info[GPIO_CONTROL_2880_PIN4].pid = 0x0;
//	tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIORENA));
//	tmp |= (0x1 << GPIO_CONTROL_2880_PIN4);
//	*(volatile u32 *)(RALINK_REG_PIORENA) = cpu_to_le32(tmp);
	tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIOFENA));
	tmp |= (0x1 << GPIO_CONTROL_2880_PIN4);
	*(volatile u32 *)(RALINK_REG_PIOFENA) = cpu_to_le32(tmp);

       tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
       tmp |= (1L << GPIO_CONTROL_2880_PIN1);
       tmp |= (1L << GPIO_CONTROL_2880_PIN2);
       tmp &= ~(1L << GPIO_CONTROL_2880_PIN3);
       *(volatile u32 *)(RALINK_REG_PIODIR) = cpu_to_le32(tmp);

       tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
       tmp |= (1L << GPIO_CONTROL_2880_PIN1);
       tmp |= (1L << GPIO_CONTROL_2880_PIN2);
      	*(volatile u32 *)(RALINK_REG_PIODATA) = cpu_to_le32(tmp);

	iNIC_piodata = (le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA))) &  RALINK_GPIO_CONTROL_RT2880_MASK_0;
	iNIC_piodata_tmp = iNIC_piodata;

#endif
#ifdef CONFIG_RALINK_GPIO_LED
	ralink_gpio_led_init_timer();
#endif

#if FOR_COREGA
	//register led data
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].gpio = PWR_LED_GPIO_PIN;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].on = 5;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].off = 5;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].blinks = 10;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].rests = 0;
	ralink_gpio_led_data[PWR_LED_GPIO_PIN].times = 10;

	//clear previous led status
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ticks = -1;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].ons = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].offs = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].resting = 0;
	ralink_gpio_led_stat[PWR_LED_GPIO_PIN].times = 0;
#endif
#ifdef SUPPORT_HW_WD
	ralink_gpio_led_data[WDI_GPIO_PIN].gpio = WDI_GPIO_PIN;
	ralink_gpio_led_data[WDI_GPIO_PIN].on = 10;
	ralink_gpio_led_data[WDI_GPIO_PIN].off = 10;
	ralink_gpio_led_data[WDI_GPIO_PIN].blinks = 10;
	ralink_gpio_led_data[WDI_GPIO_PIN].rests = 0;
	ralink_gpio_led_data[WDI_GPIO_PIN].times = 10;

	//clear previous led status
	ralink_gpio_led_stat[WDI_GPIO_PIN].ticks = -1;
	ralink_gpio_led_stat[WDI_GPIO_PIN].ons = 0;
	ralink_gpio_led_stat[WDI_GPIO_PIN].offs = 0;
	ralink_gpio_led_stat[WDI_GPIO_PIN].resting = 0;
	ralink_gpio_led_stat[WDI_GPIO_PIN].times = 0;
#endif
#ifdef iNIC_2880
	RTMP_IO_WRITE32_T(RALINK_11N_MAC_BASE, 0x102c,0x40031e46);
#else
	RTMP_IO_WRITE32_T(RALINK_11N_MAC_BASE, 0x102c,0x00031e46);
#endif
	le32_to_cpu(*(volatile u_long *)(RALINK_PIO_BASE + 0x20)) |= (1 << WPS_LED_GPIO_PIN);

	printk("Ralink gpio driver initialized\n");
	return 0;
}

void __exit ralink_gpio_exit(void)
{
#ifdef  CONFIG_DEVFS_FS
	devfs_unregister_chrdev(ralink_gpio_major, RALINK_GPIO_DEVNAME);
	devfs_unregister(devfs_handle);
#else
	unregister_chrdev(ralink_gpio_major, RALINK_GPIO_DEVNAME);
#endif

	//config these pins to normal mode
	*(volatile u32 *)(RALINK_REG_GPIOMODE) &= ~RALINK_GPIOMODE_DFT;
	//disable gpio interrupt
	*(volatile u32 *)(RALINK_REG_INTDIS) = cpu_to_le32(RALINK_INTCTL_PIO);
#ifdef CONFIG_RALINK_GPIO_LED
	del_timer(&ralink_gpio_led_timer);
#endif
        kfree(reset2def_timer);
#if FOR_IODATA
        kfree(wps_timer);
#endif
	printk("Ralink gpio driver exited\n");
}

#ifdef iNIC_2880
void iNIC_gpio_notify_user(void)
{
	struct task_struct *p1 = NULL;

	p1 = find_task_by_pid(ralink_gpio_info[GPIO_CONTROL_2880_PIN4].pid);
	if (NULL == p1) {
		printk(KERN_ERR NAME ": no registered process to notify\n");
		return;
	}
	send_sig(SIGUSR2, p1, 0);


}
#endif
/*
 * send a signal(SIGUSR1) to the registered user process whenever any gpio
 * interrupt comes
 * (called by interrupt handler)
 */
void ralink_gpio_notify_user(int usr)
{
	struct task_struct *p = NULL;

	if (ralink_gpio_irqnum < 0 || RALINK_GPIO_NUMBER <= ralink_gpio_irqnum) {
		printk(KERN_ERR NAME ": gpio irq number out of range\n");
		return;
	}

	//don't send any signal if pid is 0 or 1
	if ((int)ralink_gpio_info[ralink_gpio_irqnum].pid < 2)
		return;
	p = find_task_by_pid(ralink_gpio_info[ralink_gpio_irqnum].pid);
	if (NULL == p) {
		printk(KERN_ERR NAME ": no registered process to notify\n");
		return;
	}

	if (usr == 1) {
		//printk(KERN_NOTICE NAME ": sending a SIGUSR1 to process %d\n",
//				ralink_gpio_info[ralink_gpio_irqnum].pid);
		send_sig(SIGUSR1, p, 0);
	}
	else if (usr == 2) {
// 		printk(KERN_NOTICE NAME ": sending a SIGUSR2 to process %d\n",
// 				ralink_gpio_info[ralink_gpio_irqnum].pid);
		send_sig(SIGUSR2, p, 0);
	}
}

/*
 * 1. save the PIOINT and PIOEDGE value
 * 2. clear PIOINT by writing 1
 * (called by interrupt handler)
 */
void ralink_gpio_save_clear_intp(void)
{
	ralink_gpio_intp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIOINT));
	ralink_gpio_edge = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIOEDGE));
	*(volatile u32 *)(RALINK_REG_PIOINT) = cpu_to_le32(0x00FFFFFF);
	*(volatile u32 *)(RALINK_REG_PIOEDGE) = cpu_to_le32(0x00FFFFFF);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void ralink_gpio_irq_handler(unsigned int irq, struct irqaction *irqaction)
#else
irqreturn_t ralink_gpio_irq_handler(int irq, void *irqaction)
#endif
{
	struct gpio_time_record {
		unsigned long falling;
		unsigned long rising;
	};
	static struct gpio_time_record record[RALINK_GPIO_NUMBER];
	unsigned long now;
	int i;

	ralink_gpio_save_clear_intp();
	now = jiffies;
	for (i = 0; i < RALINK_GPIO_NUMBER; i++) {
		if (! (ralink_gpio_intp & (1 << i)))
			continue;
		ralink_gpio_irqnum = i;
#ifdef iNIC_2880
		if(ralink_gpio_irqnum == GPIO_CONTROL_2880_PIN4)
		{
			if(ralink_gpio_info[GPIO_CONTROL_2880_PIN4].pid != 0)
			{
				iNIC_piodata = (le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA))) &  RALINK_GPIO_CONTROL_RT2880_MASK_0;

				iNIC_piodata_tmp = iNIC_piodata;
				iNIC_gpio_notify_user();

			}
		}
#endif
#if 1 /* wps use interrupt */
#ifdef CONFIG_RAETH_RT3052_P5 
		if(ralink_gpio_irqnum == GPIO_CONTROL_2880_PIN1)
		{
#if 0
		   /* get WPS event from RT2880 */
		   if((RT3052_WPS_IN_PROCESS == 0) &&(RT2880_WPS_IN_PROCESS == 0))
		   {
		      if(((*(volatile u32 *)(RALINK_REG_PIODATA)) & (1L<<GPIO_CONTROL_2880_PIN2)) == 0x0)
		        RT2880_WPS_IN_PROCESS = 1;
		      //else
		       // RT2880_WPS_IN_PROCESS = 0;
		   }
           else if((RT3052_WPS_IN_PROCESS==0) && (RT2880_WPS_IN_PROCESS == 1))
           {
                le32_to_cpu(*(volatile u_long *)(RALINK_PIO_BASE + 0x20)) |= (1 << WPS_LED_GPIO_PIN);
           }
		   else
           {
			  RT2880_WPS_IN_PROCESS = 0;
          }
#else
           /* get WPS event from RT2880 */
           if(RT3052_WPS_IN_PROCESS == 0)
           {
              if(((*(volatile u32 *)(RALINK_REG_PIODATA)) & (1L<<GPIO_CONTROL_2880_PIN2)) == 0x0)
                RT2880_WPS_IN_PROCESS = 1;
              //else
               // RT2880_WPS_IN_PROCESS = 0;
           }
           else
           {
            RT2880_WPS_IN_PROCESS = 0;
//                 le32_to_cpu(*(volatile u_long *)(RALINK_PIO_BASE + 0x20)) |= (1 << WPS_LED_GPIO_PIN);
           }
#endif
		}
#endif
#endif
		if (ralink_gpio_edge & (1 << i)) { //rising edge
#if FOR_IODATA
			  del_timer(wps_timer);
#endif
			if (record[i].rising != 0 && time_before_eq(now,
						record[i].rising + 60L)) {
				/*
				 * If the interrupt comes in a short period,
				 * it might be floating. We ignore it.
				 */
			        if(detect_reset2def && (!reset2def_start))
			        {
					del_timer(reset2def_timer);
					detect_reset2def = 0;
				}
			}
			else {
				record[i].rising = now;
                                if(ralink_gpio_irqnum == WPS_GPIO_PIN)
				{
			           wps_time = (now - record[i].falling)/HZ;
				}
#ifdef SUPPORT_TWO_BOTTON
#ifdef FOR_ZYXEL
				if (now < record[i].falling + HZ) {
#elif defined(FOR_IODATA)
                		if (now > record[i].falling + (HZ*2) && (now < record[i].falling + (HZ*10)) ) {
#else
				if (now < record[i].falling + (HZ*2)) {
#endif
                                     if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                                     {
					  ralink_gpio_notify_user(1);
                                     }
#ifndef FOR_ZYXEL
				     else if(ralink_gpio_irqnum == RESET2DEF_GPIO_PIN)
                                     {
				          ralink_reset(RALINK_GPIO_RESET_PIN);
                                     }
#endif
                                     else
                                     {
                		          printk("irqnum = %d ; Not found\n",ralink_gpio_irqnum);
				     }
				}
#ifdef FOR_ZYXEL
				else if ((now >= record[i].falling + HZ) && (now < record[i].falling + (HZ*5))) {
#else
				else if ((now >= record[i].falling + (HZ*2)) && (now < record[i].falling + (HZ*10))) {
#endif
 				      if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                                      {
					  ralink_gpio_notify_user(1);
                                      }
#ifdef FOR_ZYXEL
				      else if(ralink_gpio_irqnum == RESET2DEF_GPIO_PIN)
                                      {
				          ralink_reset(RALINK_GPIO_RESET_PIN);
                                      }
#endif
				}
#ifdef FOR_ZYXEL
				else if (now >= record[i].falling + (HZ*5)) {
#else
                		else if (now >= record[i].falling + (HZ*10)) {
#endif
                                     if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                                     {
					  ralink_gpio_notify_user(1);
                                     }
				     else if(ralink_gpio_irqnum == RESET2DEF_GPIO_PIN)
                                     {
                                          reset2def_start = 1;
					  ralink_gpio_notify_user(2);
                                          break;
                                     }
                                     else
                                     {
                		          printk("irqnum = %d ; Not found\n",ralink_gpio_irqnum);
				     }
				}

				if(detect_reset2def && (!reset2def_start))
				{
					del_timer(reset2def_timer);
					detect_reset2def = 0;
				}
#else /* ! SUPPORT_TWO_BOTTON */
				if (now < record[i].falling + (time_period1*HZ)) {
                                     if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                                     {
					  ralink_gpio_notify_user(1);
                                     }
				}
#if 0
                else if ((now > record[i].falling + (time_period1*HZ)) && (now < record[i].falling + (time_period2*HZ))) 
                {
                    if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                    {
                      ralink_reset(RALINK_GPIO_RESET_PIN);
                    }
                }
#else
                else if ((now > record[i].falling + (time_period2*HZ)) && (now < record[i].falling + (time_period3*HZ))) 
                {
                    if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                    {
                      ralink_reset(RALINK_GPIO_RESET_PIN);
                    }
                }
#endif
				else if (now > record[i].falling + (time_period3*HZ)) {

 				      if(ralink_gpio_irqnum == WPS_GPIO_PIN)
                                      {
                                          reset2def_start = 1;
					  ralink_gpio_notify_user(2);
                                          break;
                                      }
				}

				if(now && (!reset2def_start))
				{

			            if (now > record[i].falling + 60L) {
					del_timer(reset2def_timer);
					detect_reset2def = 0;
				    }
				}
#endif
			}
		}
		else { //falling edge
			record[i].falling = now;
#if FOR_IODATA
			if(ralink_gpio_irqnum == WPS_GPIO_PIN)
			   add_wps_job_to_timer();
#endif
#ifdef SUPPORT_TWO_BOTTON
                        if(ralink_gpio_irqnum == RESET2DEF_GPIO_PIN)
#else
                        if(ralink_gpio_irqnum == WPS_GPIO_PIN)
#endif
                        {
                          if((!detect_reset2def) && (!reset2def_start))
                          {

                               if((record[i].falling - record[i].rising) > 12L)
			       {
			           add_reset2def_job_to_timer();
#if (HAS_INIC && FOR_SITECOM)	
	
			           wps_time2 = 0;
#endif
		               }
		                   detect_reset2def = 1;
                          }
                        }
		}
		break;
	}

	return IRQ_HANDLED;

}

struct irqaction ralink_gpio_irqaction = {
	.handler = ralink_gpio_irq_handler,
	.flags = SA_INTERRUPT,
	.mask = 0,
	.name = "ralink_gpio",
};

void __init ralink_gpio_init_irq(void)
{
	setup_irq(SURFBOARDINT_GPIO, &ralink_gpio_irqaction);
}

module_init(ralink_gpio_init);
module_exit(ralink_gpio_exit);

